package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.dto.GameDTO;
import com.zanxiang.manage.domain.dto.GameUserRoleDTO;
import com.zanxiang.manage.domain.params.UserOrderListParam;
import com.zanxiang.manage.domain.vo.GameCategoryVO;
import com.zanxiang.manage.domain.vo.UserOrderListVO;
import com.zanxiang.manage.service.GameCategoryService;
import com.zanxiang.manage.service.GameService;
import com.zanxiang.manage.service.GameUserRoleService;
import com.zanxiang.manage.service.OrderService;
import com.zanxiang.mybatis.entity.Order;
import com.zanxiang.mybatis.mapper.OrderMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author : lingfeng
 * @time : 2022-07-05
 * @description : 用户订单
 */
@Slf4j
@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements OrderService {

    @Autowired
    private GameUserRoleService gameUserRoleService;

    @Autowired
    private GameService gameService;

    @Autowired
    private GameCategoryService gameCategoryService;

    /**
     * 获取用户下单记录
     *
     * @param param : 参数
     * @return : 返回单页数据
     */
    @Override
    public IPage<UserOrderListVO> orderList(UserOrderListParam param) {
        //执行查询
        return page(param.toPage(), new QueryWrapper<Order>().lambda()
                .eq(Order::getUserId, param.getUserId())
                .orderByDesc(Order::getCreateTime)
        ).convert(this::toVo);
    }

    /**
     * 订单信息对象转换
     *
     * @param order : 订单信息
     * @return : 订单信息展示对象
     */
    private UserOrderListVO toVo(Order order) {
        if (order == null) {
            return null;
        }
        UserOrderListVO orderListVO = BeanUtils.copy(order, UserOrderListVO.class);
        //查询角色名称
        GameUserRoleDTO gameUserRoleDTO = gameUserRoleService.getByRoleId(order.getRoleId());
        if (gameUserRoleDTO != null) {
            orderListVO.setRoleName(gameUserRoleDTO.getRoleName());
        }
        //设置游戏名称
        GameDTO gameDTO = gameService.getById(order.getGameId());
        if (gameDTO == null) {
            return orderListVO;
        }
        //设置游戏列表
        GameCategoryVO gameCategoryVO = gameCategoryService.getById(gameDTO.getCategory());
        if (gameCategoryVO != null) {
            orderListVO.setGameCategoryName(gameCategoryVO.getName());
        }
        orderListVO.setGameName(gameDTO.getName());
        return orderListVO;
    }
}
